'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const __chunk_1 = require('./chunk-ec5441dd.js');

class IIIFGallery {
    constructor(hostRef) {
        __chunk_1.registerInstance(this, hostRef);
        this.items = null;
        this.selectManifest = __chunk_1.createEvent(this, "selectManifest", 7);
        this.selectCollection = __chunk_1.createEvent(this, "selectCollection", 7);
        this.selectCanvas = __chunk_1.createEvent(this, "selectCanvas", 7);
    }
    manifestHandler() {
        this._reset();
    }
    ignoreHandler() {
        this._reset();
    }
    componentWillLoad() {
        this._reset();
    }
    _reset() {
        if (this.manifest) {
            manifesto.loadManifest(this.manifest).then((data) => {
                const manifest = manifesto.create(data);
                // if it's a collection, list either the child collections or child manifests.
                if (manifest.isCollection()) {
                    this.items = manifest.items;
                }
                else {
                    // if it's a manifest, list the child canvases.
                    const sequences = manifest.getSequences();
                    if (sequences.length) {
                        const sequence = sequences[0];
                        this.items = sequence.getCanvases();
                    }
                }
                if (this.ignore) {
                    // remove any whitespace in ignore csv
                    const ignore = this.ignore.split(',').map((item) => item.trim());
                    this.items = this.items.filter(resource => ignore.indexOf(resource.id) === -1);
                }
            }).catch(function (e) {
                console.error(e);
                console.error('failed to load manifest');
            });
        }
        else {
            this.items = null;
        }
    }
    async reset() {
        this._reset();
    }
    render() {
        if (!this.items) {
            return (__chunk_1.h("span", null));
        }
        else {
            return (__chunk_1.h("section", { class: "cf w-100 pa2" }, this.items.map((item) => __chunk_1.h("iiif-gallery-item", { item: item, selected: this._selectedItem === item }))));
        }
    }
    itemSelected(event) {
        const item = event.detail;
        this._selectedItem = item;
        if (item.isCollection()) {
            this.selectCollection.emit(item);
        }
        else if (item.isManifest()) {
            this.selectManifest.emit(item);
        }
        else {
            this.selectCanvas.emit(item);
        }
    }
    static get watchers() { return {
        "manifest": ["manifestHandler"],
        "ignore": ["ignoreHandler"]
    }; }
    static get style() { return ""; }
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var classnames = createCommonjsModule(function (module) {
/*!
  Copyright (c) 2017 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/
/* global define */

(function () {

	var hasOwn = {}.hasOwnProperty;

	function classNames () {
		var classes = [];

		for (var i = 0; i < arguments.length; i++) {
			var arg = arguments[i];
			if (!arg) continue;

			var argType = typeof arg;

			if (argType === 'string' || argType === 'number') {
				classes.push(arg);
			} else if (Array.isArray(arg) && arg.length) {
				var inner = classNames.apply(null, arg);
				if (inner) {
					classes.push(inner);
				}
			} else if (argType === 'object') {
				for (var key in arg) {
					if (hasOwn.call(arg, key) && arg[key]) {
						classes.push(key);
					}
				}
			}
		}

		return classes.join(' ');
	}

	if (module.exports) {
		classNames.default = classNames;
		module.exports = classNames;
	} else {
		window.classNames = classNames;
	}
}());
});

class IIIFGalleryItem {
    constructor(hostRef) {
        __chunk_1.registerInstance(this, hostRef);
        this.selected = false;
        this.selectItem = __chunk_1.createEvent(this, "selectItem", 7);
    }
    renderLabel() {
        let label = this.item.getDefaultLabel();
        if (label) {
            return (__chunk_1.h("div", { class: "ph2 ph0-ns pb3 link" }, __chunk_1.h("h3", { class: "f5 f4-ns mb0 black-90" }, label)));
        }
    }
    render() {
        let thumbnail = this.item.getThumbnail();
        const classes = classnames({
            'collection': this.item.isCollection(),
            'manifest': this.item.isManifest(),
            'canvas': this.item.isCanvas(),
            'selected': this.selected,
            'aspect-ratio': true,
            'aspect-ratio--1x1': true
        });
        return (__chunk_1.h("div", { onClick: () => this._itemSelectedHandler(), class: "fl w-100 w-50-m  w-25-ns pa2" }, __chunk_1.h("div", { class: classes }, (thumbnail) ? __chunk_1.h("img", { src: thumbnail.id, class: "db bg-center cover aspect-ratio--object" }) : __chunk_1.h("span", null, "No thumbnail")), this.renderLabel()));
    }
    _itemSelectedHandler() {
        this.selectItem.emit(this.item);
    }
    static get style() { return ""; }
}

exports.iiif_gallery = IIIFGallery;
exports.iiif_gallery_item = IIIFGalleryItem;
