import { h } from '@stencil/core';
export class IIIFGallery {
    constructor() {
        this.items = null;
    }
    manifestHandler() {
        this._reset();
    }
    ignoreHandler() {
        this._reset();
    }
    componentWillLoad() {
        this._reset();
    }
    _reset() {
        if (this.manifest) {
            manifesto.loadManifest(this.manifest).then((data) => {
                const manifest = manifesto.create(data);
                // if it's a collection, list either the child collections or child manifests.
                if (manifest.isCollection()) {
                    this.items = manifest.items;
                }
                else {
                    // if it's a manifest, list the child canvases.
                    const sequences = manifest.getSequences();
                    if (sequences.length) {
                        const sequence = sequences[0];
                        this.items = sequence.getCanvases();
                    }
                }
                if (this.ignore) {
                    // remove any whitespace in ignore csv
                    const ignore = this.ignore.split(',').map((item) => item.trim());
                    this.items = this.items.filter(resource => ignore.indexOf(resource.id) === -1);
                }
            }).catch(function (e) {
                console.error(e);
                console.error('failed to load manifest');
            });
        }
        else {
            this.items = null;
        }
    }
    async reset() {
        this._reset();
    }
    render() {
        if (!this.items) {
            return (h("span", null));
        }
        else {
            return (h("section", { class: "cf w-100 pa2" }, this.items.map((item) => h("iiif-gallery-item", { item: item, selected: this._selectedItem === item }))));
        }
    }
    itemSelected(event) {
        const item = event.detail;
        this._selectedItem = item;
        if (item.isCollection()) {
            this.selectCollection.emit(item);
        }
        else if (item.isManifest()) {
            this.selectManifest.emit(item);
        }
        else {
            this.selectCanvas.emit(item);
        }
    }
    static get is() { return "iiif-gallery"; }
    static get originalStyleUrls() { return {
        "$": ["iiif-gallery.css"]
    }; }
    static get styleUrls() { return {
        "$": ["iiif-gallery.css"]
    }; }
    static get properties() { return {
        "manifest": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "manifest",
            "reflect": false
        },
        "ignore": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "ignore",
            "reflect": false
        }
    }; }
    static get states() { return {
        "items": {}
    }; }
    static get events() { return [{
            "method": "selectManifest",
            "name": "selectManifest",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "selectCollection",
            "name": "selectCollection",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "selectCanvas",
            "name": "selectCanvas",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }]; }
    static get methods() { return {
        "reset": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "",
                "tags": []
            }
        }
    }; }
    static get watchers() { return [{
            "propName": "manifest",
            "methodName": "manifestHandler"
        }, {
            "propName": "ignore",
            "methodName": "ignoreHandler"
        }]; }
    static get listeners() { return [{
            "name": "selectItem",
            "method": "itemSelected",
            "target": undefined,
            "capture": false,
            "passive": false
        }]; }
}
